// -*- C++ -*-
//
// user_ana.cc
//
// Author:  Mitsuhiro YAMAGA
// Created: Sat Oct 12 00:48:09 JST 2002
//
// $Id: user_ana.cc,v 1.2 2002/10/11 17:01:47 yamaga Exp $
//
// Revision History
//
// $Log: user_ana.cc,v $
// Revision 1.2  2002/10/11 17:01:47  yamaga
// updates.
//
// Revision 1.1  2002/10/11 16:39:40  yamaga
// new eventChecker code.
//
//
//

#include <iostream>
#include <iomanip>
#include <unistd.h>
#include "E391Detector/E391Detector.h"

//
extern int  Debug;
extern int  A0evnumSlot;
extern int  A1evnumSlot;
extern int  T0evnumSlot;
extern int  T1evnumSlot;
extern int  A0evnumChannel;
extern int  A1evnumChannel;
extern int  T0evnumChannel;
extern int  T1evnumChannel;

extern E391Detector&  e391det;

// init routine
void
user_init()
{
  // Histogram definition
}


// begin_of_run routine
void
user_begin_of_run( int run_number )
{
  // 
}


// process_event routine
void
user_process_event( int run_number, int spill_number, int event_number )
{
  std::cout << "Run" << run_number
            << " spill=" << spill_number
            << " event=" << event_number;
    
  
  // # of event in the buffer
  int  nb,neSMP0,neSMP1,neNGF0,neNGF1;
  neSMP0 = neSMP1 = neNGF0 = neNGF1 = 0;

  DWORD*  pdata;
  nb = e391det.bank( "SMP0",&pdata );
  if ( nb >= 3 && pdata != 0 ){
    neSMP0 = pdata[1] & 0xffff;
  }
  nb = e391det.bank( "SMP1",&pdata );
  if ( nb >= 3 && pdata != 0 ){
    neSMP1 = pdata[1] & 0xffff;
  }
  nb = e391det.bank( "NGF0",&pdata );
  if ( nb > 0 && pdata != 0 ){
    neNGF0 = pdata[0];
  }
  nb = e391det.bank( "NGF1",&pdata );
  if ( nb > 0 && pdata != 0 ){
    neNGF1 = pdata[0];
  }

  // dump
  std::cout << " : Nevent :"
            << " SMP0=" << neSMP0 
            << " SMP1=" << neSMP1 
            << " NGF0=" << neNGF0 
            << " NGF1=" << neNGF1;
  
  if ( neSMP0 == neSMP1 && neSMP1 == neNGF0 && neNGF0 == neNGF1 ){
    std::cout << "   OK" << std::endl;
  }
  else{
    std::cout << " ## SLIP ##" << std::endl;
  }
  
  //
  int  tdc0evnum,tdc1evnum,adc0evnum,adc1evnum;
  tdc0evnum = e391det.tdcCrate(0).tdc( T0evnumSlot ).hitPattern() 
    >> T0evnumChannel;
  
  tdc1evnum = e391det.tdcCrate(1).tdc( T1evnumSlot ).hitPattern() 
    >> T1evnumChannel;
  
  if ( A0evnumChannel < 48 )
    adc0evnum = e391det.adcCrate(0).adc( A0evnumSlot ).hitPatternL(1000) 
      >> A0evnumChannel;
  else
    adc0evnum = e391det.adcCrate(0).adc( A0evnumSlot ).hitPatternH(1000) 
      >> (A0evnumChannel-48);

  if ( A1evnumChannel < 48 )
    adc1evnum = e391det.adcCrate(1).adc( A1evnumSlot ).hitPatternL(1000) 
      >> A1evnumChannel;
  else
    adc1evnum = e391det.adcCrate(1).adc( A1evnumSlot ).hitPatternH(1000) 
      >> (A1evnumChannel-48);
   //
  std::cout << " EventNumber :"
            << " TDC(0:" << T0evnumSlot << ":" << T0evnumChannel << ")="
            << std::hex << std::setw(2) << std::setfill('0')
            << tdc0evnum
            << std::dec
            << " TDC(1:" << T1evnumSlot << ":" << T1evnumChannel << ")="
            << std::hex << std::setw(2) << std::setfill('0')
            << tdc1evnum
            << std::dec
            << " ADC(0:" << A0evnumSlot << ":" << A0evnumChannel << ")="
            << std::hex << std::setw(2) << std::setfill('0')
            << adc0evnum
            << std::dec
            << " ADC(1:" << A1evnumSlot << ":" << A1evnumChannel << ")="
            << std::hex << std::setw(2) << std::setfill('0')
            << adc1evnum
            << std::dec;

  //
  if ( tdc0evnum == tdc1evnum && 
       tdc0evnum == adc0evnum &&
       tdc0evnum == adc1evnum ){
    std::cout << "   OK" << std::endl;
  }
  else{
    std::cout << " ## SLIP ##" << std::endl;
  }
  
}


// process_event routine
void
user_process_event_exp( HNDLE hbuf,HNDLE request_id,
                        EVENT_HEADER* pheader, void* pevent )
{
  /* ODB structures */
  extern RUNINFO  runinfo;
  
  int  spillnum = pheader->serial_number;

  //
  static int  RunNumber = 0;
  if ( RunNumber != runinfo.run_number ){
    RunNumber = runinfo.run_number;
    e391det.setRunNumber( RunNumber );
    // get DB for this run
    e391det.getDB();
  }

  //
  e391det.setEvent( pheader, pevent );
  DWORD*  pdata;
  int  nb = e391det.bank( "NGF0",&pdata );
  
  if ( nb > 0 ){
    if ( e391det.unpack( 1 ) == 0 ){
      user_process_event( RunNumber,spillnum,1 );
    }
  }
}


// end_of_run routine
void
user_end_of_run( int run_number )
{
  //
}


// term routine
void
user_term()
{
  //
}


