// -*- C++ -*-
//
// user_ana.cc
//
// Author:  Ken Sakashita
// Created: Mon Oct  7 22:51:58 JST 2002
//
// $Id: user_ana.cc,v 1.1 2002/10/07 14:04:50 kensh Exp $
//
// $Log: user_ana.cc,v $
// Revision 1.1  2002/10/07 14:04:50  kensh
//
//
// Add new package "onlineMonitor"
//
//
//
//

#include <iostream>
#include <iomanip>
#include <unistd.h>
#include "E391Detector/E391Detector.h"
#include "ccfortran/ccfortran.h"

#define  TRIG_SPILLON  0x01
#define  TRIG_CLOCK    0x02
#define  TRIG_COSMIC   0x04
#define  TRIG_XE       0x08
#define  TRIG_NCLUS    0x10

#define  SCL_SPILL     0
#define  SCL_REQUEST   1
#define  SCL_ACCEPT    2
#define  SCL_SEC       9
#define  SCL_TMON      10
#define  SCL_CLOCK     12
#define  SCL_COSMIC    13
#define  SCL_XE        14
#define  SCL_NCLUS     15
   
#define  MAX_ADC_CH    96
#define  MAX_TDC_CH    64

//
extern E391Detector&  e391det;
extern unsigned int  TriggerBit;

//
int AdcCutH  = 580;
int AdcCutL  = 500;

//
// For RunControl
static int oldnRequestEvent;
static int oldnAcceptEvent;
static int oldnSEC;
static int oldnTMON;
//
static int oldnClockEvent;
static int oldnCosmicEvent;
static int oldnXeEvent;
static int oldnNclsEvent;
//
static int initRequestEvent = 0;
static int initAcceptEvent  = 0;

//
static bool sethist = false;
// M.Yamaga
static bool  init = false;

// Higz
#define NWPAWC  1000000
int  pawc_[NWPAWC];

// init routine
void
user_init()
{
  // init HIGZ
  //HLIMIT( NWPAWC );
  HLIMAP( NWPAWC, "ONLN" );

  if( !sethist ) { 
    HDELET( 0 );
    
    //
    HBOOK1(  1, "N clock per spill", 1, 0., 1.);
    HBOOK1(  2, "N cosmic per spill", 1, 0., 1.);
    HBOOK1(  3, "N Xe per spill", 1, 0., 1.);
    HBOOK1(  4, "N Cluster per spill", 1, 0., 1.);
    HBOOK1(  5, "Trigger efficiency per spill", 1, 0., 1.);
    HBOOK1(  6, "Trigger efficiency averaged", 1, 0., 1.);
    HBOOK1(  7, "N SEC per spill", 1, 0., 1.);
    HBOOK1(  8, "N TMON per spill", 1, 0., 1.);
    
    // For DAQ Status
    HBOOK1(  10, "N Event   : SMP0", 2000, 0., 2000.);
    HBOOK1(  20, "N Event   : SMP1", 2000, 0., 2000.);
    HBOOK1(  30, "N Event   : NGF0", 2000, 0., 2000.);
    HBOOK1(  40, "N Event   : NGF1", 2000, 0., 2000.);

    sethist = true;

  }

}


// begin_of_run routine
void
user_begin_of_run( int run_number )
{
  //
  if( !sethist ) {
    HDELET( 0 );
    
    //
    HBOOK1(  1, "N clock per spill", 1, 0., 1.);
    HBOOK1(  2, "N cosmic per spill", 1, 0., 1.);
    HBOOK1(  3, "N Xe per spill", 1, 0., 1.);
    HBOOK1(  4, "N Cluster per spill", 1, 0., 1.);
    HBOOK1(  5, "Trigger efficiency per spill", 1, 0., 1.);
    HBOOK1(  6, "Trigger efficiency averaged", 1, 0., 1.);
    HBOOK1(  7, "N SEC per spill", 1, 0., 1.);
    HBOOK1(  8, "N TMON per spill", 1, 0., 1.);
    
    // For DAQ Status
    HBOOK1(  10, "N Event   : SMP0", 2000, 0., 2000.);
    HBOOK1(  20, "N Event   : SMP1", 2000, 0., 2000.);
    HBOOK1(  30, "N Event   : NGF0", 2000, 0., 2000.);
    HBOOK1(  40, "N Event   : NGF1", 2000, 0., 2000.);
    
    sethist = true;
  }
}


// process_event routine
void
user_process_event( int run_number, int spill_number,
                    int event_number, int trigger_bit )
{

}


// process_event routine
void
user_process_event_exp( HNDLE hbuf,HNDLE request_id,
                        EVENT_HEADER* pheader, void* pevent )
{
  /* ODB structures */
  extern RUNINFO  runinfo;
  
  int  spillnum = pheader->serial_number;

  //
  static int  RunNumber = 0;
  if ( RunNumber != runinfo.run_number ){
    RunNumber = runinfo.run_number;
    e391det.setRunNumber( RunNumber );
    // get DB for this run
    e391det.getDB();
    //e391det.getDB( 1 );
  }

  //
  e391det.setEvent( pheader, pevent );
  int iSpill = e391det.spillNumber();

  // Scaler event
  DWORD*  pdataSCL0;
  int scl0size = e391det.bank( "SCL0", &pdataSCL0 );
  if ( scl0size > 0 ) {
    if ( !init ){
      initRequestEvent = pdataSCL0[ SCL_REQUEST ];
      initAcceptEvent  = pdataSCL0[ SCL_ACCEPT  ];
      init = true;
    }
    
    int RequestEvent  = pdataSCL0[ SCL_REQUEST ] - initRequestEvent;
    int AcceptEvent   = pdataSCL0[ SCL_ACCEPT  ] - initAcceptEvent;
    int nRequestEvent = pdataSCL0[ SCL_REQUEST ] - oldnRequestEvent;
    int nAcceptEvent  = pdataSCL0[ SCL_ACCEPT  ] - oldnAcceptEvent;
    int nSEC          = pdataSCL0[ SCL_SEC     ] - oldnSEC;
    int nTMON         = pdataSCL0[ SCL_TMON    ] - oldnTMON;
    
    int nClockEvent   = pdataSCL0[ SCL_CLOCK   ] - oldnClockEvent;
    int nCosmicEvent  = pdataSCL0[ SCL_COSMIC  ] - oldnCosmicEvent;
    int nXeEvent      = pdataSCL0[ SCL_XE      ] - oldnXeEvent;
    int nNclsEvent    = pdataSCL0[ SCL_NCLUS   ] - oldnNclsEvent;
    

    HRESET( 1, " ");
    HRESET( 2, " ");
    HRESET( 3, " ");
    HRESET( 4, " ");
    HRESET( 5, " ");
    HRESET( 6, " ");

    HFILL(  1, 0.5, 0., (float)nClockEvent);
    HFILL(  2, 0.5, 0., (float)nCosmicEvent);
    HFILL(  3, 0.5, 0., (float)nXeEvent);
    HFILL(  4, 0.5, 0., (float)nNclsEvent);
    if( nRequestEvent != 0 ) 
      HFILL(  5, 0.5, 0., (float)nAcceptEvent/nRequestEvent);
    if( RequestEvent != 0 ) // M.Yamaga
      HFILL(  6, 0.5, 0., (float)AcceptEvent/RequestEvent );

    // DAQ stability 
    if( iSpill%1000 == 0 ) {
      // DAQ status
      HBOOK1(  50, "# of request event per spill", 1000, (float)iSpill, (float)(iSpill + 1000) );
      HBOOK1(  51, "integrated # of request event", 1000, (float)iSpill, (float)(iSpill + 1000) );
      HBOOK1(  52, "# of accept event per spill", 1000, (float)iSpill, (float)(iSpill + 1000) );
      HBOOK1(  53, "integrated # of accept event", 1000, (float)iSpill, (float)(iSpill + 1000) );
      HBOOK1(  54, "Trigger efficiency per spill", 1000, (float)iSpill, (float)(iSpill + 1000) );
      HBOOK1(  55, "Trigger efficiency", 1000, (float)iSpill, (float)(iSpill + 1000) );

    }
    if( !HEXIST(50) ) {
      int xmin = ( iSpill - iSpill%1000 );
      int xmax = xmin + 1000;
      HBOOK1(  50, "# of request event per spill", 1000, (float)xmin, (float)xmax );
    }
    if( !HEXIST(51) ) {
      int xmin = ( iSpill - iSpill%1000 );
      int xmax = xmin + 1000;
      HBOOK1(  51, "integrated # of request event", 1000, (float)xmin, (float)xmax );
    }
    if( !HEXIST(52) ) {
      int xmin = ( iSpill - iSpill%1000 );
      int xmax = xmin + 1000;
      HBOOK1(  52, "# of accept event per spill", 1000, (float)xmin, (float)xmax );
    }
    if( !HEXIST(53) ) {
      int xmin = ( iSpill - iSpill%1000 );
      int xmax = xmin + 1000;
      HBOOK1(  53, "integrated # of accept event", 1000, (float)xmin, (float)xmax );
    }
    if( !HEXIST(54) ) {
      int xmin = ( iSpill - iSpill%1000 );
      int xmax = xmin + 1000;
      HBOOK1(  54, "Trigger efficiency per spill", 1000, (float)xmin, (float)xmax );
    }
    if( !HEXIST(55) ) {
      int xmin = ( iSpill - iSpill%1000 );
      int xmax = xmin + 1000;
      HBOOK1(  55, "Trigger efficiency", 1000, (float)xmin, (float)xmax );
    }
    HFILL( 50, 0.5 + (float)iSpill, 0., (float)nRequestEvent);
    HFILL( 51, 0.5 + (float)iSpill, 0., (float)RequestEvent);
    HFILL( 52, 0.5 + (float)iSpill, 0., (float)nAcceptEvent);
    HFILL( 53, 0.5 + (float)iSpill, 0., (float)AcceptEvent);
    if( nRequestEvent != 0 ) 
      HFILL( 54, 0.5 + (float)iSpill, 0., (float)nAcceptEvent/(float)nRequestEvent);
    if( RequestEvent != 0 ) 
      HFILL( 55, 0.5 + (float)iSpill, 0., (float)AcceptEvent/(float)RequestEvent);
 
    //////////////////
    //
    oldnRequestEvent = pdataSCL0[ SCL_REQUEST ];
    oldnAcceptEvent  = pdataSCL0[ SCL_ACCEPT  ];
    oldnSEC          = pdataSCL0[ SCL_SEC     ];
    oldnTMON         = pdataSCL0[ SCL_TMON    ];
    //
    oldnClockEvent   = pdataSCL0[ SCL_CLOCK   ];
    oldnCosmicEvent  = pdataSCL0[ SCL_COSMIC  ];
    oldnXeEvent      = pdataSCL0[ SCL_XE      ];
    oldnNclsEvent    = pdataSCL0[ SCL_NCLUS   ];

  }

  // Data event
  DWORD*  pdata;
  int  nb = e391det.bank( "NGF0",&pdata );
  if ( nb > 0 ){
    std::cout << "Run" << runinfo.run_number
              << " spill=" << spillnum
              << " nevent=" << pdata[0]
              << std::endl;

    int  nevent = pdata[0];
    int  nev = 1;
    int  ct = 0;

    do {
      if ( e391det.unpack( nev ) == 0 ){
        int  trig = e391det.triggerBit(nev);

	if ( trig&TRIG_CLOCK ) { // clock event
	  // NGF0
	  const std::list<AdcModule>& adc0List = e391det.adcCrate( 0 ).adcList();
	  for( std::list<AdcModule>::const_iterator m = adc0List.begin();
	       m != adc0List.end(); m++ ){
	    int hid = 1100 + m->pa();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "Pedestal : NGF0 : ADC PA = %02d", m->pa());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_ADC_CH; ch++ ) {
	      int rangeBit = m->data( ch )&0xffff0000;
	      int rawAdc   = m->data( ch )&0xffff;

	      if( !rangeBit ) HFILL( hid, 0.5 + (float)ch, 0., (float)rawAdc ); 
	    }
	  } 

	  
	}
	else if ( trig&TRIG_COSMIC ) { // Cosmic event
	  // NGF0
	  const std::list<AdcModule>& adc0List = e391det.adcCrate( 0 ).adcList();
	  for( std::list<AdcModule>::const_iterator m = adc0List.begin();
	       m != adc0List.end(); m++ ){
	    int hid = 2100 + m->pa();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "CosmicTrig : Hit pattern : NGF0 : ADC PA = %02d", m->pa());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_ADC_CH; ch++ ) {
	      int rangeBit = m->data( ch )&0xffff0000;
	      int rawAdc   = m->data( ch )&0xffff;
	      int hit      = ( rangeBit ? rawAdc - AdcCutH : rawAdc - AdcCutL);

	      if( hit > 0 ) HFILL( hid, 0.5 + (float)ch, 0., 1. ); 
	    }

	  }
	  // NGF1
	  const std::list<AdcModule>& adc1List = e391det.adcCrate( 1 ).adcList();
	  for( std::list<AdcModule>::const_iterator m = adc1List.begin();
	       m != adc1List.end(); m++ ){
	    int hid = 2200 + m->pa();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "CosmicTrig : Hit pattern : NGF1 : ADC PA = %02d", m->pa());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_ADC_CH; ch++ ) {
	      int rangeBit = m->data( ch )&0xffff0000;
	      int rawAdc   = m->data( ch )&0xffff;
	      int hit      = ( rangeBit ? rawAdc - AdcCutH : rawAdc - AdcCutL);

	      if( hit > 0 ) HFILL( hid, 0.5 + (float)ch, 0., 1. ); 
	    }

	  }
	  // SMP0
	  const std::list<TdcModule>& tdc0List = e391det.tdcCrate( 0 ).tdcList();
	  for( std::list<TdcModule>::const_iterator m = tdc0List.begin();
	       m != tdc0List.end(); m++ ){
	    int hid = 2300 + m->ma();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "CosmicTrig : Hit pattern : SMP0 : TDC MA = %02d", m->ma());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_TDC_CH; ch++ ) {
	      int rawTdc   = m->data( ch );
	      if( rawTdc > 0 ) HFILL( hid, 0.5 + (float)ch, 0., 1. ); 

	    }

	  }
	  // SMP1
	  const std::list<TdcModule>& tdc1List = e391det.tdcCrate( 1 ).tdcList();
	  for( std::list<TdcModule>::const_iterator m = tdc1List.begin();
	       m != tdc1List.end(); m++ ){
	    int hid = 2400 + m->ma();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "CosmicTrig : Hit pattern : SMP1 : TDC MA = %02d", m->ma());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_TDC_CH; ch++ ) {
	      int rawTdc   = m->data( ch );
	      if( rawTdc > 0 ) HFILL( hid, 0.5 + (float)ch, 0., 1. ); 

	    }

	  }


	}
	else if ( trig&TRIG_XE ) { // Xe event
	  // NGF0
	  const std::list<AdcModule>& adc0List = e391det.adcCrate( 0 ).adcList();
	  for( std::list<AdcModule>::const_iterator m = adc0List.begin();
	       m != adc0List.end(); m++ ){
	    int hid = 3100 + m->pa();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "XeTrig : Hit pattern : NGF0 : ADC PA = %02d", m->pa());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_ADC_CH; ch++ ) {
	      int rangeBit = m->data( ch )&0xffff0000;
	      int rawAdc   = m->data( ch )&0xffff;
	      int hit      = ( rangeBit ? rawAdc - AdcCutH : rawAdc - AdcCutL);

	      if( hit > 0 ) HFILL( hid, 0.5 + (float)ch, 0., 1. ); 
	    }

	  }
	  // NGF1
	  const std::list<AdcModule>& adc1List = e391det.adcCrate( 1 ).adcList();
	  for( std::list<AdcModule>::const_iterator m = adc1List.begin();
	       m != adc1List.end(); m++ ){
	    int hid = 3200 + m->pa();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "XeTrig : Hit pattern : NGF1 : ADC PA = %02d", m->pa());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_ADC_CH; ch++ ) {
	      int rangeBit = m->data( ch )&0xffff0000;
	      int rawAdc   = m->data( ch )&0xffff;
	      int hit      = ( rangeBit ? rawAdc - AdcCutH : rawAdc - AdcCutL);

	      if( hit > 0 ) HFILL( hid, 0.5 + (float)ch, 0., 1. ); 
	    }

	  }
	  // SMP0
	  const std::list<TdcModule>& tdc0List = e391det.tdcCrate( 0 ).tdcList();
	  for( std::list<TdcModule>::const_iterator m = tdc0List.begin();
	       m != tdc0List.end(); m++ ){
	    int hid = 3300 + m->ma();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "XeTrig : Hit pattern : SMP0 : TDC MA = %02d", m->ma());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_TDC_CH; ch++ ) {
	      int rawTdc   = m->data( ch );
	      if( rawTdc > 0 ) HFILL( hid, 0.5 + (float)ch, 0., 1. ); 

	    }

	  }
	  // SMP1
	  const std::list<TdcModule>& tdc1List = e391det.tdcCrate( 1 ).tdcList();
	  for( std::list<TdcModule>::const_iterator m = tdc1List.begin();
	       m != tdc1List.end(); m++ ){
	    int hid = 3400 + m->ma();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "XeTrig : Hit pattern : SMP1 : TDC MA = %02d", m->ma());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_TDC_CH; ch++ ) {
	      int rawTdc   = m->data( ch );
	      if( rawTdc > 0 ) HFILL( hid, 0.5 + (float)ch, 0., 1. ); 

	    }

	  }



	}
	else if ( trig&TRIG_NCLUS ) { // Nclus event
	  // NGF0
	  const std::list<AdcModule>& adc0List = e391det.adcCrate( 0 ).adcList();
	  for( std::list<AdcModule>::const_iterator m = adc0List.begin();
	       m != adc0List.end(); m++ ){
	    int hid = 4100 + m->pa();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "NclusterTrig : Hit pattern : NGF0 : ADC PA = %02d", m->pa());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_ADC_CH; ch++ ) {
	      int rangeBit = m->data( ch )&0xffff0000;
	      int rawAdc   = m->data( ch )&0xffff;
	      int hit      = ( rangeBit ? rawAdc - AdcCutH : rawAdc - AdcCutL);

	      if( hit > 0 ) HFILL( hid, 0.5 + (float)ch, 0., 1. ); 
	    }

	  }
	  // NGF1
	  const std::list<AdcModule>& adc1List = e391det.adcCrate( 1 ).adcList();
	  for( std::list<AdcModule>::const_iterator m = adc1List.begin();
	       m != adc1List.end(); m++ ){
	    int hid = 4200 + m->pa();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "NclusterTrig : Hit pattern : NGF1 : ADC PA = %02d", m->pa());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_ADC_CH; ch++ ) {
	      int rangeBit = m->data( ch )&0xffff0000;
	      int rawAdc   = m->data( ch )&0xffff;
	      int hit      = ( rangeBit ? rawAdc - AdcCutH : rawAdc - AdcCutL);

	      if( hit > 0 ) HFILL( hid, 0.5 + (float)ch, 0., 1. ); 
	    }

	  }
	  // SMP0
	  const std::list<TdcModule>& tdc0List = e391det.tdcCrate( 0 ).tdcList();
	  for( std::list<TdcModule>::const_iterator m = tdc0List.begin();
	       m != tdc0List.end(); m++ ){
	    int hid = 4300 + m->ma();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "NclusterTrig : Hit pattern : SMP0 : TDC MA = %02d", m->ma());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_TDC_CH; ch++ ) {
	      int rawTdc   = m->data( ch );
	      if( rawTdc > 0 ) HFILL( hid, 0.5 + (float)ch, 0., 1. ); 

	    }

	  }
	  // SMP1
	  const std::list<TdcModule>& tdc1List = e391det.tdcCrate( 1 ).tdcList();
	  for( std::list<TdcModule>::const_iterator m = tdc1List.begin();
	       m != tdc1List.end(); m++ ){
	    int hid = 4400 + m->ma();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "NclusterTrig : Hit pattern : SMP1 : TDC MA = %02d", m->ma());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_TDC_CH; ch++ ) {
	      int rawTdc   = m->data( ch );
	      if( rawTdc > 0 ) HFILL( hid, 0.5 + (float)ch, 0., 1. ); 

	    }

	  }
	  //
	}
	nev++;

      }
      else{
        break;
      }

    }while( nev <= nevent );

  } // End of Data event

}


// end_of_run routine
void
user_end_of_run( int run_number )
{
  //
  HDELET( 0 );
  sethist = false;
  //init    = false;
}


// term routine
void
user_term()
{
  //
}


