// -*- C++ -*-
//
// command_loop.cc
//
// Author:  Mitsuhiro YAMAGA
// Created: Thu Oct 10 16:15:05 JST 2002
//
// $Id: command_loop.cc,v 1.1 2002/10/10 16:40:18 yamaga Exp $
//
// Revision History
//
// $Log: command_loop.cc,v $
// Revision 1.1  2002/10/10 16:40:18  yamaga
// new package
//
//
//

#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "midas.h"
#include "msystem.h"

//
static char data[50000];

//
typedef struct {
  int flags;
  char pattern[32];
  int index;
} PRINT_INFO;

#define PI_LONG      (1<<0)
#define PI_RECURSIVE (1<<1)
#define PI_VALUE     (1<<2)
#define PI_HEX       (1<<3)
#define PI_PAUSE     (1<<4)


//
void
compose_name(char *pwd, char *name, char *full_name)
{
  if (name[0] != '/')
    {
    strcpy(full_name, pwd);
    if (full_name[strlen(full_name)-1] != '/')
      strcat(full_name, "/");
    strcat(full_name, name);
    }
  else
    strcpy(full_name, name);
}


// process a wildcard match
BOOL
match(char *pat, char *str)
{    
  if (!*str)
    return *pat == '*' ? match (pat+1, str) : !*pat;    

  switch (*pat)    
    {
    case '\0': 
      return 0;
    case '*' : 
      return match(pat+1, str) || match(pat, str+1);
    case '?' : 
      return match(pat+1, str+1);
    default  : 
      return (toupper(*pat) == toupper(*str)) && match(pat+1, str+1);    
    }
}


//
INT
print_key(HNDLE hDB, HNDLE hKey, KEY *key, INT level, void *info)
{
  INT         i, size, status;
  static char data_str[10000], line[256];
  DWORD       delta;
  PRINT_INFO  *pi;

  pi = (PRINT_INFO *) info;

  /* if pattern set, check if match */
  if ( pi->pattern[0] && !match(pi->pattern, key->name) )
    return SUCCESS;
  
  size = sizeof(data);
  if (pi->flags & PI_VALUE){
    /* only print value */
    if (key->type != TID_KEY){
      status = db_get_data(hDB, hKey, data, &size, key->type);
      if (status == DB_NO_ACCESS)
        strcpy(data_str, "<no read access>");
      else{
        for (i=0 ; i<key->num_values ; i++){
          if (pi->flags & PI_HEX)
            db_sprintfh(data_str, data, key->item_size, i, key->type);
          else
            db_sprintf(data_str, data, key->item_size, i, key->type);
          
          if ( (pi->index != -1 && i == pi->index) ||
               pi->index == -1 )
            printf("%s\n", data_str);
        }
      }
    }
  }
  else {
    /* print key name with value */
    memset(line, ' ', 80);
    line[80] = 0;
    sprintf(line + level*4, "%s", key->name);
    if (pi->index != -1)
      sprintf(line + strlen(line), "[%d]", pi->index);
    line[strlen(line)] = ' ';
    
    if (key->type == TID_KEY){
      if (pi->flags & PI_LONG)
        sprintf(line+32, "DIR"); 
      else
        line[32] = 0;
      printf("%s\n", line);
    }
    else {
      status = db_get_data(hDB, hKey, data, &size, key->type);
      if (status == DB_NO_ACCESS)
        strcpy(data_str, "<no read access>");
      else {
        if (pi->flags & PI_HEX)
          db_sprintfh(data_str, data, key->item_size, 0, key->type);
        else
          db_sprintf(data_str, data, key->item_size, 0, key->type);
      }
      
      if (pi->flags & PI_LONG){
        sprintf(line+32, "%s", rpc_tid_name(key->type));
        line[strlen(line)] = ' ';
        sprintf(line+40, "%ld", key->num_values);
        line[strlen(line)] = ' ';
        sprintf(line+46, "%ld", key->item_size);
        line[strlen(line)] = ' ';
        
        db_get_key_time(hDB, hKey, &delta);
        if (delta < 60)
          sprintf(line+52, "%ds", delta);
        else if (delta < 3600)
          sprintf(line+52, "%1.0lfm", delta/60.0);
        else if (delta < 86400)
          sprintf(line+52, "%1.0lfh", delta/3600.0);
        else if (delta < 86400*99)
          sprintf(line+52, "%1.0lfh", delta/86400.0);
        else
          sprintf(line+52, ">99d");
        line[strlen(line)] = ' ';
        
        sprintf(line+57, "%d", key->notify_count);
        line[strlen(line)] = ' ';
        
        if (key->access_mode & MODE_READ)
          line[61] = 'R';
        if (key->access_mode & MODE_WRITE)
          line[62] = 'W';
        if (key->access_mode & MODE_DELETE)
          line[63] = 'D';
        if (key->access_mode & MODE_EXCLUSIVE)
          line[64] = 'E';
        
        if (key->type == TID_STRING && strchr(data_str, '\n'))
          strcpy(line+66, "<multi-line>");
        else if (key->num_values == 1)
          strcpy(line+66, data_str);
        else
          line[66] = 0;
      }
      else{
        if (key->num_values == 1){
          if (key->type == TID_STRING && strchr(data_str, '\n'))
            strcpy(line+32, "<multi-line>");
          else
            strcpy(line+32, data_str);
        }
        else{
          line[32] = 0;
        }
      }
      printf("%s\n", line);
      
      if (key->type == TID_STRING && strchr(data_str, '\n'))
        puts(data_str);
      
      if (key->num_values > 1){
        for (i=0 ; i<key->num_values ; i++){
          if (pi->flags & PI_HEX)
            db_sprintfh(data_str, data, key->item_size, i, key->type);
          else
            db_sprintf(data_str, data, key->item_size, i, key->type);
          
          memset(line, ' ', 80);
          line[80] = 0;
          
          if (pi->flags & PI_LONG){
            sprintf(line+40, "[%ld]", i);
            line[strlen(line)] = ' ';
            
            strcpy(line+56, data_str);
          }
          else
            strcpy(line+32, data_str);
          
          if ((pi->index != -1 && i == pi->index) ||
              pi->index == -1)
            printf("%s\n", line);
          
        }
      }
    }
  }
    
  return SUCCESS;
}


//
void
set_key(HNDLE hDB, HNDLE hKey, int index1, int index2, char *value)
{
  KEY  key;
  char data[1000];
  int  i, size, status;

  db_get_key(hDB, hKey, &key);

  memset(data, 0, sizeof(data));
  db_sscanf(value, data, &size, 0, key.type);

  /* extend data size for single string if necessary */
  if ((key.type == TID_STRING || key.type == TID_LINK) 
        && (int) strlen(data)+1 > key.item_size &&
      key.num_values == 1)
    key.item_size = strlen(data)+1;

  if (key.item_size == 0)
    key.item_size = rpc_tid_size(key.type);

  if (key.num_values > 1 && index1 == -1)
    {
    for (i=0 ; i<key.num_values ; i++)
      status = db_set_data_index(hDB, hKey, data, key.item_size, i, key.type);
    }
  else if (key.num_values > 1 && index2>index1)
    {
    for (i=index1 ; i<key.num_values && i<=index2; i++)
      status = db_set_data_index(hDB, hKey, data, key.item_size, i, key.type);
    }
  else if (key.num_values > 1 || index1 > 0)
    status = db_set_data_index(hDB, hKey, data, key.item_size, index1, key.type);
  else
    status = db_set_data(hDB, hKey, data, key.item_size, 1, key.type);

  if (status == DB_NO_ACCESS)
    printf("Write access not allowed\n");
}

//
void
command_loop( char *host_name, char *exp_name )
{
  HNDLE  hDB, hKey, hKeyClient, hSubkey, hRootKey;
  KEY    key;
  PRINT_INFO  print_info;

  cm_get_experiment_database(&hDB, &hKeyClient);

  int  debug_flag = 0;
  
  while(1){
    //
    std::cout << "runControl > ";
    std::cout.flush();

    //
    char  ss[1024];
    fgets( ss,1024,stdin );

    //
    char  cmdstr[16][128];
    char*  si = ss;
    char*  se = si;
    int  ncmd = 0;
    while( *si != 0 && ncmd < 16 ){
      if ( *si == '\'' ){
        se = ++si;
        while( *se != '\'' && *se != 0 )
          se++;
        *se = 0;
        strncpy( cmdstr[ncmd++],si,128 );
        si = se + 1;
      }
      else if ( *si == '"' ){
        se = ++si;
        while( *se != '"' && *se != 0 )
          se++;
        *se = 0;
        strncpy( cmdstr[ncmd++],si,128 );
        si = se + 1;
      }
      else if ( *si == ' '  || *si == '\t' ||
                *si == '\r' || *si == '\n' ){
        si++;
      }
      else{
        se = si + 1;
        while( *se != ' '  && *se != '\t' &&
               *se != '\r' && *se != '\n'  )
          se++;
        *se = 0;
        strncpy( cmdstr[ncmd++],si,128 );
        si = se + 1;
      }
    }
    for( int i=ncmd;i<16;i++ ){
      cmdstr[i][0] = 0;
    }
    
    // q(uit)
    if ( cmdstr[0][0] == 'q' ){
      break;
    }

    // start
    else if ( strcmp( cmdstr[0],"start" )==0 ){
      int  ipar = STATE_STOPPED;
      int  size = sizeof(int);
      db_get_value(hDB, 0, "/Runinfo/State",
                   &ipar, &size, TID_INT, TRUE);
      if ( ipar == STATE_RUNNING ){
        std::cout << "Run is already started."  << std::endl;
      }
      else if ( ipar == STATE_PAUSED ){
        std::cout << "Run is paused. Please use \"resume\""  << std::endl;
      }
      else{
        ipar = 0;
        db_get_value(hDB, 0, "/Runinfo/Transition in progress",
                     &ipar, &size, TID_INT, TRUE);
        if ( ipar == 1 ){
          std::cout << "Start/Stop in transition."  << std::endl;
        }
        else{
          int  run_number = 0;
          db_get_value(hDB, 0, "/Runinfo/Run number", 
                       &run_number, &size, TID_INT, TRUE);
          run_number++;
          ipar = 1;
          db_set_value(hDB, 0, "/Runinfo/Transition in progress",
                       &ipar,  sizeof(INT), 1, TID_INT);
          char  str[256] = "";
          int  status = cm_transition(TR_START, run_number, str, sizeof(str),
                                      SYNC, debug_flag);
          if (status != CM_SUCCESS){
            run_number--;
            db_set_value(hDB, 0, "/Runinfo/Run number", 
                         &run_number, sizeof(int), 1, TID_INT);
            std::cout << "Error: " << str << std::endl;
          }
          ipar = 0;
          db_set_value(hDB, 0, "/Runinfo/Transition in progress",
                       &ipar, sizeof(INT), 1, TID_INT);
        }
      }
    }

    // stop
    else if ( strcmp( cmdstr[0],"stop" )==0 ){
      int  ipar = 0;
      int  size = sizeof(int);
      db_get_value(hDB, 0, "/Runinfo/Transition in progress",
                   &ipar, &size, TID_INT, TRUE);
      if ( ipar != 1 ){
        ipar = 1;
        db_set_value(hDB, 0, "/Runinfo/Transition in progress",
                     &ipar,  sizeof(INT), 1, TID_INT);
        char  str[256] = "";
        int  status = cm_transition(TR_STOP, 0, str, sizeof(str),
                                    SYNC, debug_flag);
        if ( status == CM_DEFERRED_TRANSITION ){
          std::cout << str << std::endl;
        }
        else if ( status == CM_TRANSITION_IN_PROGRESS ){
          std::cout << "Deferred stop already in progress." << std::endl;
        }
        else if ( status != CM_SUCCESS ){
          std::cout << "Error: " << str << std::endl;
        }
        ipar = 0;
        db_set_value(hDB, 0, "/Runinfo/Transition in progress",
                     &ipar, sizeof(int), 1, TID_INT);
      }
    }

    // pause
    else if ( strcmp( cmdstr[0],"pause" )==0 ){
      // check if run is started
      int  ipar = STATE_STOPPED;
      int  size = sizeof(ipar);
      db_get_value(hDB, 0, "/Runinfo/State", 
                   &ipar, &size, TID_INT, TRUE);
      if ( ipar != STATE_RUNNING ){
        std::cout << "Run is not started." << std::endl;
      }
      else{
        char  str[256] = "";
        int  status = cm_transition(TR_PAUSE, 0, str, sizeof(str),
                                    SYNC, debug_flag);
        if ( status == CM_DEFERRED_TRANSITION ){
          std::cout << str << std::endl;
        }
        else if ( status == CM_TRANSITION_IN_PROGRESS ){
          std::cout << "Deferred pause already in progress." << std::endl;
        }
        else if ( status != CM_SUCCESS ){
          std::cout << "Error: " << str << std::endl;
        }
      }
    }

    // resume
    else if ( strcmp( cmdstr[0],"resume" )==0 ){
      // check if run is paused
      int  ipar = STATE_STOPPED;
      int  size = sizeof(ipar);
      db_get_value(hDB, 0, "/Runinfo/State", 
                   &ipar, &size, TID_INT, TRUE);
      if ( ipar != STATE_PAUSED ){
        std::cout << "Run is not paused." << std::endl;
      }
      else{
        char  str[256] = "";
        int  status = cm_transition(TR_RESUME, 0, str, sizeof(str),
                                    SYNC, debug_flag);
        if ( status != CM_SUCCESS ){
          std::cout << "Error: " << str << std::endl;
        }
      }
    }

    // ls
    else if ( strcmp( cmdstr[0],"ls" )==0 ){
      print_info.flags = 0;
      print_info.pattern[0] = 0;
      /* parse options */
      for( int i=1; i<16; i++ ){
        if ( cmdstr[i][0] == '-' ){
          for ( int j=1; cmdstr[i][j]!=' ' && cmdstr[i][j] ; j++ ){
            if (cmdstr[i][j] == 'l')
              print_info.flags |= PI_LONG;
            if (cmdstr[i][j] == 'r')
              print_info.flags |= PI_RECURSIVE;
            if (cmdstr[i][j] == 'v')
              print_info.flags |= PI_VALUE;
            if (cmdstr[i][j] == 'h')
              print_info.flags |= PI_HEX;
            if (cmdstr[i][j] == 'p')
              print_info.flags |= PI_PAUSE;
          }
        }
      }
      print_info.index = -1;
      //
      for( int i=1; i<16; i++ ){
        if ( cmdstr[i][0] == 0 ){
          if ( i==1 )
            strcpy( cmdstr[i],"/" );
          else
            break;
        }
        else if ( cmdstr[i][0] == '-' ){
          continue;
        }
        
        int  status = db_find_key( hDB, 0, cmdstr[i], &hKey );
        if (status == DB_NO_KEY)
          std::cout << "key " << cmdstr[i] << " not found." << std::endl;
        if (status == DB_INVALID_LINK)
          std::cout << "link " << cmdstr[i]
                    << " points to invalid location." << std::endl;
        if ( hKey ){
          if ( (print_info.flags & PI_LONG) && 
               (print_info.flags & PI_VALUE)==0 ){
            printf( "Key name                        Type    #Val  Size  Last Opn Mode Value\n");
            printf("---------------------------------------------------------------------------\n");
          }
          if (print_info.flags & PI_RECURSIVE){
            db_scan_tree(hDB, hKey, 0, print_key, &print_info);
            if (cm_is_ctrlc_pressed())
              cm_ack_ctrlc_pressed();
          }
          else{
            db_get_key(hDB, hKey, &key);
            if ( key.type != TID_KEY )
              print_key(hDB, hKey, &key, 0, &print_info);
            else{
              for( int i=0; ; i++ ){
                if (cm_is_ctrlc_pressed()) {
                  cm_ack_ctrlc_pressed();
                  break;
                }
                
                db_enum_link(hDB, hKey, i, &hSubkey);
                
                if (!hSubkey)
                  break;
                
                db_get_key(hDB, hSubkey, &key);
                status = print_key(hDB, hSubkey, &key, 0, &print_info);
                if ( status == 0 )
                  break;
              }
            }
          }
        }
      }
    }

    // set
    else if ( strcmp( cmdstr[0],"set" )==0 ){
      /* check if index is supplied */
      int  index1 = 0, index2 = 0;
      char  str[256];
      strcpy( str, cmdstr[1] );
      if ( str[strlen(str)-1] == ']' ){
        if ( strchr(str, '[') ){
          if ( *(strchr(str, '[')+1) == '*' )
            index1 = -1;
          else if ( strchr((strchr(str, '[')+1), '.') ){
            index1 = atoi( strchr(str, '[') + 1 );
            char*  pc = strchr(str, '[')+1;
            while (*pc != '.')
              pc++;
            while (*pc == '.')
              pc++;
            index2 = atoi(pc);
          }
          else
            index1 = atoi(strchr(str, '[')+1);
        }
        *strchr(str, '[') = 0;
      }

      if (strpbrk(str, "*?") != NULL){
        db_find_key(hDB, 0, "/", &hKey);
        for ( int i=0; ; i++){
          db_enum_link(hDB, hKey, i, &hSubkey);
          
          if (!hSubkey)
            break;
          
          db_get_key(hDB, hSubkey, &key);
          if (match(str, key.name))
            set_key(hDB, hSubkey, index1, index2, cmdstr[2]);
        }
      }
      else {
        char  name[256];
        compose_name("/", str, name);
        
        int  status = db_find_key(hDB, 0, name, &hKey);
        if (status == DB_SUCCESS)
          set_key(hDB, hKey, index1, index2, cmdstr[2]);
        else {
          printf("key not found\n");
        }
      }
    }

    // shutdowm
    else if ( strcmp( cmdstr[0],"shutdown" )==0 ){
      if ( cmdstr[1][0] == 0 ){
        std::cout << "Please enter client name "
                  << "or \"all\" to shutdown all clients."
                  << std::endl;
      }
      else{
        int  status = cm_shutdown( cmdstr[1], TRUE );
        if ( status != CM_SUCCESS ){
          if ( strcmp( cmdstr[1], "all" ) == 0 )
            std::cout << "No clients found." << std::endl;
          else
            std::cout << "Client " << cmdstr[1] 
                      << " not active" << std::endl;
        }
      }
    }

    // scl
    else if ( strcmp( cmdstr[0],"scl" )==0 ){
      int  status = db_find_key(hDB, 0, "System/Clients", &hKey);
      if (status != DB_SUCCESS)
        cm_msg(MERROR, "command_loop", "cannot find System/Clients entry in database");
      else {
        if ( cmdstr[1][1] == 'w')
          printf("Name                Host                Timeout    Last called\n");
        else
          printf("Name                Host\n");

        /* search database for clients with transition mask set */
        for ( int i=0,status=0; ; i++ ) {
          status = db_enum_key(hDB, hKey, i, &hSubkey);
          if (status == DB_NO_MORE_SUBKEYS)
            break;

          if (status == DB_SUCCESS){
            char  name[256];
            int  size = sizeof(name);
            db_get_value(hDB, hSubkey, "Name", name, &size, TID_STRING, TRUE);
            printf(name);
            for ( int j=0; j<20-(int)strlen(name); j++ )
              printf(" ");

            char  str[256];
            size = sizeof(str);
            db_get_value(hDB, hSubkey, "Host", str, &size, TID_STRING, TRUE);
            printf(str);
            for ( int j=0; j<20-(int)strlen(str); j++ )
              printf(" ");

            /* display optional watchdog info */
            if ( cmdstr[1][1] == 'w' ){
              INT timeout, last;
              
              status = cm_get_watchdog_info(hDB, name, &timeout, &last);
              printf("%-10d %-10d", timeout, last);
            }

            printf("\n");
          }
        }
      }
    }
 
    // check if client connections are broken
    int  status = cm_yield(0);
    if (status == SS_ABORT)
      break;
  }

}
