// -*- C++ -*-
//
// GsimHits.cc
// Detector hit recording function
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Wed Jul 11 15:29:57 JST 2001
//
// $Id: GsimHits.cc,v 1.1.1.1 2002/08/13 12:48:08 kensh Exp $
//
// Revision History
//
// $Log: GsimHits.cc,v $
// Revision 1.1.1.1  2002/08/13 12:48:08  kensh
//
//
// Start version.
//
//

#include <iostream>
#include <string>
#include <math.h>
#include "ccfortran/G321Common.h"
#include "ccfortran/ccfortran.h"
#include "CLHEP/Vector/ThreeVector.h"
#include "gsim-general/GsimGeneral.h"
#include "gsim-general/Ntuple.h"
#include "ntuple.h"


//////
#define  DEBUG  0


// GsimUserGeom.cc
extern int    DetUseFBAR[];
extern int    DetUseCBAR[];
extern int    DetUseCSIC[];
extern int    DetUseCSEG[];
extern int    DetUseCC01[];
extern int    DetUseCC02[];
extern int    DetUseCC03[];
extern int    DetUseCC04[];
extern int    DetUseCC05[];
extern int    DetUseBMCT[];
extern int    DetUseCHGV[];


//////
int
GsimHits()
{
#if DEBUG
  std::cerr << "GsimHits" << std::endl;
#endif

  // Return code. 0:Do not stop, >0:Stop swimming, <0:Stop Event_process
  int  stop = 0;

  // Check Volume
  std::string  dname( (char*)&gcvolu_.NAMES[gcvolu_.NLEVEL-1],4 );
#if DEBUG
  std::cerr << dname << " : edep=" << gctrak_.DESTEP << std::endl;
#endif

  int     pid  = gckine_.IPART;
  int     lv   = gcvolu_.NLEVEL - 1;
  int     id   = gcvolu_.NUMBER[lv]-1;
  int     moid = gcvolu_.NUMBER[lv-1]-1; // ID of mother volume
  double  ekin = gctrak_.GEKIN;
  double  etot = gctrak_.GETOT;
  double  edep = gctrak_.DESTEP;
  Hep3Vector  v( gctrak_.VECT[0],gctrak_.VECT[1],gctrak_.VECT[2] );
  Hep3Vector  p( gctrak_.VECT[3]*gctrak_.VECT[6],
                 gctrak_.VECT[4]*gctrak_.VECT[6],
                 gctrak_.VECT[5]*gctrak_.VECT[6] );

  // CsI
  if ( dname == "CSIC" || dname == "CSDT" || dname == "CSDB" ){
    if ( DetUseCSIC[1] ){
      nresult.ECSI[id] += edep;
    }
    else{ // Do not produce shower : stop
      nresult.ECSI[id] += ekin;
      
      // pile up handling
      if ( nresult.PNCSIHIT[id] ){
        std::cout << "GsimHits : Event" << gcflag_.IDEVT
                  << " : CsI hit piled up : crystal=" << id 
                  << " : prev_pid=" << nresult.PNCSIHIT[id]
                  << " : new_pid=" << pid
                  << std::endl;
      }
      nresult.XCSIHIT[id]  = ( nresult.XCSIHIT[id]*nresult.EKCSIHIT[id] 
                               + gctrak_.VECT[0]*ekin )
                             / ( nresult.EKCSIHIT[id]+ekin );
      nresult.YCSIHIT[id]  = ( nresult.YCSIHIT[id]*nresult.EKCSIHIT[id] 
                               + gctrak_.VECT[1]*ekin )
                             / ( nresult.EKCSIHIT[id]+ekin );
      nresult.ZCSIHIT[id]  = ( nresult.ZCSIHIT[id]*nresult.EKCSIHIT[id] 
                               + gctrak_.VECT[2]*ekin )
                             / ( nresult.EKCSIHIT[id]+ekin );
      nresult.PXCSIHIT[id] = gctrak_.VECT[3];
      nresult.PYCSIHIT[id] = gctrak_.VECT[4];
      nresult.PZCSIHIT[id] = gctrak_.VECT[5];
      nresult.PPCSIHIT[id] = gctrak_.VECT[6];
      nresult.EKCSIHIT[id] += ekin;
      nresult.PNCSIHIT[id] = nresult.PNCSIHIT[id]*100 + pid;
      //

      stop = 2;
    }
#if DEBUG
    std::cerr << "GsimHits : CrystalID=" << id 
              << " edep=" << edep << " (" << nresult.ECSI[id] << ")" 
              << std::endl;
#endif
  }
  // CC03
  else if ( dname == "CC*S" ){
    if ( DetUseCC03[1] ){
      nresult.ECC3[moid] += edep;
    }
    else{ // Do not produce shower : stop
      nresult.ECC3[moid] += ekin;
      stop = 2;
    }
  }
  else if ( dname == "CC*W" ){
    if ( DetUseCC03[1] ){
      //
    }
    else{
      // Do not produce shower : stop
      nresult.ECC3[moid] += ekin;
      stop = 2;
    }
  }
  // Small Edge Calorimator in the CsI cylinder
  else if ( dname == "CSEG" ){
    if ( DetUseCSEG[1] ){
      nresult.ECSI[id] += edep;
    }
    else{ // Do not produce shower : stop
      nresult.ECSI[id] += ekin;

      // pile up handling, same as CsI
      if ( nresult.PNCSIHIT[id] ){
        std::cout << "GsimHits : Event" << gcflag_.IDEVT
                  << " : SEG hit piled up : det_id=" << id 
                  << " : prev_pid=" << nresult.PNCSIHIT[id]
                  << " : new_pid=" << pid
                  << std::endl;
      }
      nresult.XCSIHIT[id]  = ( nresult.XCSIHIT[id]*nresult.EKCSIHIT[id] 
                               + gctrak_.VECT[0]*ekin )
                             / ( nresult.EKCSIHIT[id]+ekin );
      nresult.YCSIHIT[id]  = ( nresult.YCSIHIT[id]*nresult.EKCSIHIT[id] 
                               + gctrak_.VECT[1]*ekin )
                             / ( nresult.EKCSIHIT[id]+ekin );
      nresult.ZCSIHIT[id]  = ( nresult.ZCSIHIT[id]*nresult.EKCSIHIT[id] 
                               + gctrak_.VECT[2]*ekin )
                             / ( nresult.EKCSIHIT[id]+ekin );
      nresult.PXCSIHIT[id] = gctrak_.VECT[3];
      nresult.PYCSIHIT[id] = gctrak_.VECT[4];
      nresult.PZCSIHIT[id] = gctrak_.VECT[5];
      nresult.PPCSIHIT[id] = gctrak_.VECT[6];
      nresult.EKCSIHIT[id] += ekin;
      nresult.PNCSIHIT[id] = nresult.PNCSIHIT[id]*100 + pid;
      //

      stop = 2;
    }
#if DEBUG
    std::cerr << "GsimHits : CrystalID=" << id 
              << " edep=" << edep << " (" << nresult.ECSI[id] << ")" 
              << std::endl;
#endif
  }
  // Charged Veto
  else if ( dname.find( "CHV" ) < dname.length() ){
    nresult.ECHV[id] += edep;
#if DEBUG
    std::cerr << dname << "[" << id << "]" << v 
              << " R=" << v.perp()
              << " phi=" << v.phi()/M_PI*180
              << " : edep=" << edep << std::endl;
#endif
  }
  // Front Barrel
  else if ( dname == "FBA1" || dname == "FBA2" ){
    nresult.EFBR[id] += ekin;
    stop = 2;
  }
  // Central Barrel
  else if ( dname == "CBAR" ){
    nresult.ECBR[id] += ekin;
    stop = 2;
  }
  // Collar counters, Beam Catcher
  else if ( dname == "CC01" ){
    nresult.ECC1 += ekin;
    stop = 2;
  }
  else if ( dname == "CC02" ){
    nresult.ECC2 += ekin;
    stop = 2;
  }
  else if ( dname == "CC04" ){
    nresult.ECC4 += ekin;
    stop = 2;
  }
  else if ( dname == "CC05" ){
    nresult.ECC5 += ekin;
    stop = 2;
  }
  else if ( dname == "BMCT" ){
    nresult.EBMC += ekin;
    stop = 2;
  }

  return( stop );
}


//////
int
GsimResults()
{
#if DEBUG
    std::cerr << "GsimHits" << std::endl;
#endif
  
  int  nhit = 0;
  
  // CsI
  int  ct = 0;
  for( int i=0;i<600;i++ ){
    if ( nresult.ECSI[i] > 0 ){
      nresult.ECSI[ct]  = nresult.ECSI[i];

      nresult.XCSIHIT[ct]  = nresult.XCSIHIT[i];
      nresult.YCSIHIT[ct]  = nresult.YCSIHIT[i];
      nresult.ZCSIHIT[ct]  = nresult.ZCSIHIT[i];
      nresult.PXCSIHIT[ct] = nresult.PXCSIHIT[i];
      nresult.PYCSIHIT[ct] = nresult.PYCSIHIT[i];
      nresult.PZCSIHIT[ct] = nresult.PZCSIHIT[i];
      nresult.PPCSIHIT[ct] = nresult.PPCSIHIT[i];
      nresult.EKCSIHIT[ct] = nresult.EKCSIHIT[i];
      nresult.PNCSIHIT[ct] = nresult.PNCSIHIT[i];

      nresult.IDCSI[ct] = i;
      ct++;
    }
  }
  nresult.NCSI = ct;
  nhit += ct;

  // FBAR
  ct = 0;
  for( int i=0;i<16;i++ ){
    if ( nresult.EFBR[i] > 0 ){
      nresult.EFBR[ct]  = nresult.EFBR[i];
      nresult.IDFBR[ct] = i;
      ct++;
    }
  }
  nresult.NFBR = ct;
  nhit += ct;
  
  // CBAR
  ct = 0;
  for( int i=0;i<32;i++ ){
    if ( nresult.ECBR[i] > 0 ){
      nresult.ECBR[ct]  = nresult.ECBR[i];
      nresult.IDCBR[ct] = i;
      ct++;
    }
  }
  nresult.NCBR = ct;
  nhit += ct;
  
  // CC03
  ct = 0;
  for( int i=0;i<4;i++ ){
    if ( nresult.ECC3[i] > 0 ){
      nresult.ECC3[ct]  = nresult.ECC3[i];
      nresult.IDCC3[ct] = i;
      ct++;
    }
  }
  nresult.NCC3 = ct;
  nhit += ct;
  
  // CHGV
  ct = 0;
  for( int i=0;i<32;i++ ){
    if ( nresult.ECHV[i] > 0 ){
      nresult.ECHV[ct]  = nresult.ECHV[i];
      nresult.IDCHV[ct] = i;
      ct++;
    }
  }
  nresult.NCHV = ct;
  nhit += ct;
  
  //
  if ( nresult.ECC1 > 0 ) nhit++;
  if ( nresult.ECC2 > 0 ) nhit++;
  if ( nresult.ECC4 > 0 ) nhit++;
  if ( nresult.ECC5 > 0 ) nhit++;
  
  // copy GEN_PARTICLE to NRESULT table
  nresult.NGEN = gen_particle.NGEN;
  for( int i=0; i<gen_particle.NGEN; i++ ){
    nresult.XX[i]   = gen_particle.XX[i];
    nresult.YY[i]   = gen_particle.YY[i];
    nresult.ZZ[i]   = gen_particle.ZZ[i];
    nresult.PX[i]   = gen_particle.PX[i];
    nresult.PY[i]   = gen_particle.PY[i];
    nresult.PZ[i]   = gen_particle.PZ[i];
    nresult.PP[i]   = gen_particle.PP[i];
    nresult.EK[i]   = gen_particle.EK[i];
    nresult.MASS[i] = gen_particle.MASS[i];
    nresult.PN[i]   = gen_particle.PN[i];
    nresult.MO[i]   = gen_particle.MO[i];
    nresult.DA1[i]  = gen_particle.DA1[i];
    nresult.DA2[i]  = gen_particle.DA2[i];
    nresult.MECH[i] = gen_particle.MECH[i];
    nresult.GSTATUS[i] = gen_particle.STATUS[i];
  }

  //
  return( nhit );
}
