// -*- C++ -*-
//
// MidasFileIO.cc
// FileIO class
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Tue Jul 10 18:40:57 JST 2001
//
// $Id: MidasFileIO.h,v 1.2 2002/08/29 13:30:28 yamaga Exp $
//
// Revision History
//
// $Log: MidasFileIO.h,v $
// Revision 1.2  2002/08/29 13:30:28  yamaga
// File format is Little-endian byte-order in any architecture.
//
// Revision 1.1.1.1  2002/08/13 12:48:12  kensh
//
//
// Start version.
//
//

#ifndef MIDASFILEIO_H_INCLUDED
#define MIDASFILEIO_H_INCLUDED

#include <string>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "zlib.h"
#include "MidasManager/MidasBank.h"


//////
#define  MIO_HPSS      1
#define  MIO_COMPRESS  2


//////
class MidasFileIO {
public:
  MidasFileIO();
  MidasFileIO( const std::string& fname, int flags, int permission=0644 );
  MidasFileIO( FILE* fp, int options=0 );
  MidasFileIO( int fd, int options=0 );
  MidasFileIO( const MidasFileIO& );
  ~MidasFileIO();
  //
  MidasFileIO&  operator=( const MidasFileIO& );
  //
  MidasBank*  bank( MidasBank& bank );
  MidasBank*  bank( MidasBank* bank );
  MidasBank*  bank() { return m_bank; }
  //
  std::string  filename()  { return m_filename;    }
  int          flags()     { return m_flags;       }
  int          permission(){ return m_permission;  }
  int          options()   { return m_options;     }
  bool         good()      { return m_good;        }

  //
  int     open( const std::string& fname, int flags=0, int permission=0644 );
  int     open( FILE* fp, int options=0 );
  int     open( int fd, int options=0 );
  int     close();
  int     readBOR( bool Rewind=false );
  int     readEvent( int event_id );
  int     skipEvent( int event_id, int n );
  int     seekEvent( int event_id, int serial, bool rewind=true );
  void    rewind();
  int     writeEvent();
  int     writeSpecialEvent(); // BOR,EOR,etc

private:
  MidasBank*   m_bank;
  MidasBank*   m_localbank;
  std::string  m_filename;
  int          m_flags;
  int          m_permission;
  int          m_options;
  int          m_fd;
  FILE*        m_fp;
  gzFile       m_zfp;
  bool         m_good;
  //
  int          m_endian;
  static MidasBank  s_writebank; // temporary write buffer
  //
  void         byte_swap_for_write_event( void* pevent );
  bool         init();
};


#endif //MIDASFILEIO_H_INCLUDED
