// -*- C++ -*-
//
// AdcEvent.cc
//
// Mon Jul 15 09:00:04 JST 2002
// Mitsuhiro YAMAGA
//
// $Id: AdcEvent.cc,v 1.1.1.1 2002/08/13 12:48:12 kensh Exp $
//
// $Log: AdcEvent.cc,v $
// Revision 1.1.1.1  2002/08/13 12:48:12  kensh
//
//
// Start version.
//
//

#include <iostream>
#include <iomanip>
#include "midas.h"
#include "unpack/AdcModule.h"
#include "unpack/AdcEvent.h"

#define DEBUG  0

static const EVENT_HEADER  zero_eh = {0,0,0,0,0};


//
AdcEvent::AdcEvent()
  : m_eventHeader( zero_eh ),
    m_serial( -1 ),
    m_crate( -1 ),
    m_valid( false ),
    m_trg( 0 )
{
  //
}


//
AdcEvent::AdcEvent( int crate, const EVENT_HEADER* pheader, const DWORD* pdata,
                    int banksize, int nth_event)
  : m_eventHeader( zero_eh ),
    m_serial( -1 ),
    m_crate( crate ),
    m_valid( false ),
    m_trg( 0 )
{
  //
  unpack( m_crate, pheader, pdata, banksize, nth_event );
}


//
AdcEvent::~AdcEvent()
{

}

//
const AdcModule&
AdcEvent::adc( int pa ) const
{
  for( std::list<AdcModule>::const_iterator i = m_adcList.begin();
       i != m_adcList.end(); i++ ){
    if ( i->pa() == pa )
      return( *i );
  }
  static const AdcModule  dummy;
  return( dummy );
}

//
unsigned int
AdcEvent::ngf( int nth ) const
{
  int  ct=0;
  for( std::list<unsigned int>::const_iterator i = m_ngf.begin();
       i != m_ngf.end(); i++ ){
    if ( ct++ == nth )
      return( *i );
  }
  return( 0 );
}

//
int
AdcEvent::unpack( int crate, const EVENT_HEADER* pheader, const DWORD* pdata,
                  int banksize, int nth_event )
{
  // remove the previous data
  m_adcList.clear();
  m_serial = -1;

  // crate
  m_crate = crate;

  // event header
  if ( pheader )
    m_eventHeader = *pheader;

  // start
  m_valid = false;
  for( int i=0;i<banksize;i++ ){
#if DEBUG
    std::cout << std::setw(8) << std::setfill('0') << std::hex 
              << pdata[i] << std::dec << std::endl;
#endif
    if ( pdata[i] == 0xffffffff ){
#if DEBUG
      std::cout << "Endmark found." << std::endl;
#endif
      break;
    }
    // skip
    int  serial = (pdata[i] >> 16 ) & 0xffff;
    int  nhit   = pdata[i] & 0xffff;
#if DEBUG
    std::cout << "Serial = " << serial << " nhit=" << nhit << std::endl;
#endif
    if ( nth_event > 0 && serial != nth_event ){
      i += nhit;
      continue;
    }
    // Nth event
    m_valid = true;
    m_serial = serial;
    AdcModule*  am = NULL;
    int  oldpa = -1;
    for( int n=0;n<nhit;n++ ){
      int  data = pdata[i+1+n];
      if ( data == 0xff000000 ){
        break;
      }
      // LRS1885N data structure
      int  pa    = (data >> 27) & 0x1f;
      int  range = (data >> 23) & 0x01;
      int  ch    = (data >> 16) & 0x7f;
      int  dt    = (data & 0xffff) | (range << 16);
      if ( pa != oldpa ){
        if ( am != NULL ){
#if DEBUG
          am->show();
#endif
          m_adcList.push_back( *am );
          delete am;
          am = NULL;
        }
        am = new AdcModule( m_crate,pa );
        oldpa = pa;
      }
      am->setData( ch,dt );
      am->hitList().push_back( data );
    }
    if ( am != NULL ){
#if DEBUG
      am->show();
#endif
      m_adcList.push_back( *am );
      delete am;
      am = NULL;
    }
    i += nhit;
  }
#if DEBUG
  std::cout << "Event end." << std::endl;
#endif

  if ( m_valid )
    return( m_adcList.size() ); // nth_event found.
  return( -1 ); // No such serial # in the spill.
}


//
int
AdcEvent::nhit() const
{
  int  n = 0;
  for( std::list<AdcModule>::const_iterator m = m_adcList.begin();
       m != m_adcList.end(); m++ ){
    n += m->hitList().size();
  }
  return( n );
}



int
AdcEvent::ngf( const DWORD* pdata, int banksize )
{
  m_ngf.clear();
  if ( 0 < banksize ){
    m_ngf.push_back( pdata[0] );
    if ( 1 < banksize && 0 < m_serial ){
      int  nstat = pdata[1];
      m_ngf.push_back( nstat );
      if ( 2 + nstat*m_serial <= banksize ){
        for( int i=0;i<nstat;i++ ){
          m_ngf.push_back( pdata[2+nstat*(m_serial-1)+i] );
        }
      }
    }
  }

  return( m_ngf.size() );
}


int
AdcEvent::trg( const DWORD* pdata, int banksize )
{
#if DEBUG
  std::cout << "AdcEvent::trg : banksize=" << banksize
            << " serial#=" << m_serial
            << " trg[" << m_serial-1 << "]=" 
            << std::hex << std::setw(8) << std::setfill('0')
            << pdata[m_serial-1]
            << std::dec
            << std::endl;
#endif
  if ( 0 < banksize && 0 <= m_serial && m_serial <= banksize ){
    return( m_trg = pdata[m_serial-1] );
  }
  return( 0 );
}

